package net.knarcraft.blacksmith.config;

import org.bukkit.Material;

/**
 * A representation of all available filters for smith presets
 */
public enum SmithPresetFilter {

    /**
     * Filters to only include items made of wood
     */
    WOOD(true, "WOODEN_"),

    /**
     * Filters to only include items made of iron
     */
    IRON(true, "IRON_"),

    /**
     * Filters to only include items made of leather
     */
    LEATHER(true, "LEATHER_"),

    /**
     * Filters to only include items made of chainmail
     */
    CHAINMAIL(true, "CHAINMAIL_"),

    /**
     * Filters to only include items made of gold
     */
    GOLD(true, "GOLDEN_"),

    /**
     * Filters to only include items made of diamond
     */
    DIAMOND(true, "DIAMOND"),

    /**
     * Filters to only include items made of netherite
     */
    NETHERITE(true, "NETHERITE_"),

    /**
     * Filters to only include items made of stone
     */
    STONE(true, "STONE_"),

    /**
     * Filters to only include bows
     */
    BOW(false, "BOW"),

    /**
     * Filters to only include pickaxes
     */
    PICKAXE(false, "_PICKAXE"),

    /**
     * Filters to only include hoes
     */
    HOE(false, "_HOE"),

    /**
     * Filters to only include shovels
     */
    SHOVEL(false, "_SHOVEL"),

    /**
     * Filters to only include axes
     */
    AXE(false, "_AXE"),

    /**
     * Filters to only include swords
     */
    SWORD(false, "_SWORD"),

    /**
     * Filters to only include ranged weapons (bows and trident)
     */
    RANGED(false, null),

    /**
     * Filters to only include miscellaneous tools (fishing rod, flint and steel, shears)
     */
    MISC(false, null);

    private final boolean searchStart;
    private final String identifier;

    /**
     * Instantiates a new smith preset filter
     *
     * @param searchStart <p>Whether to search the start of a string for the identifier</p>
     * @param identifier  <p>The string any material matching this filter contains</p>
     */
    SmithPresetFilter(boolean searchStart, String identifier) {
        this.searchStart = searchStart;
        this.identifier = identifier;
    }

    /**
     * Gets whether the given material is included in this filter
     *
     * @param material <p>The material to check</p>
     * @return <p>True if the material is included in this filter</p>
     */
    public boolean isIncluded(Material material) {
        if (isSpecialCase()) {
            return isIncludedSpecialCase(material);
        }
        if (this.searchStart) {
            return material.name().startsWith(this.identifier);
        } else {
            return material.name().endsWith(this.identifier);
        }
    }

    /**
     * Checks if this filter is a special case which doesn't work with normal rules
     *
     * @return <p>True if this filter is a special case</p>
     */
    private boolean isSpecialCase() {
        return this == RANGED || this == MISC;
    }

    /**
     * Uses a special-case check to see if the given material is part of this filter
     *
     * @param material <p>The material to check</p>
     * @return <p>True if the material is part of this filter</p>
     */
    private boolean isIncludedSpecialCase(Material material) {
        if (this == RANGED) {
            return material.name().endsWith(BOW.identifier) || material == Material.TRIDENT;
        } else if (this == MISC) {
            return material == Material.SHEARS || material == Material.FLINT_AND_STEEL ||
                    material == Material.FISHING_ROD;
        } else {
            return false;
        }
    }

}
